#include "MyWire.h"
#include <msp430.h>

void MyWire::begin(void)
{  //set GPIO for I2C
    WDTCTL = WDTPW + WDTHOLD;

   // Configure GPIO
    P1OUT &= ~BIT0;                         // Clear P1.0 output latch
    P1DIR |= BIT0;                          // For LED
    P1SEL0 |= BIT2 | BIT3;                  // I2C pins
    
   // Disable the GPIO power-on default high-impedance mode to activate
    // previously configured port settings
    PM5CTL0 &= ~LOCKLPM5;
    
     UCB0Setup();
}

void MyWire::UCB0Setup(void)
{
     // Configure USCI_B0 for I2C mode
    UCB0CTLW0 |= UCSWRST;                   // Software reset enabled
    UCB0CTLW0 |= UCMODE_3 | UCMST | UCSYNC; // I2C mode, Master mode (all 3)
    UCB0CTLW1 |= UCASTP_0;                  // No automatic STOP generation. 
                                            // 
    UCB0BRW = 0x0008;                       // baudrate = SMCLK / 8
  //  UCB0TBCNT = 0x0001;                     // number of bytes to be received ??
    UCB0I2CSA = 0x3C;                     // Slave address
    UCB0CTLW0 &= ~UCSWRST;
   // UCB0IE |= UCTXIE0;  //| UCNACKIE | UCBCNTIE;
   // UCB0CTLW0 |= UCTR | UCTXSTT;    // I2C TX, start condition
}

void MyWire::beginTransmission( unsigned char address)
{
      // UCB0Setup();
     UCB0IFG &= ~UCTXIFG; // Clear USCI_B0 TX int flag 
     UCB0IE |= UCTXIE0;  //| UCNACKIE | UCBCNTIE;
     UCB0CTLW0 |= UCTR | UCTXSTT;    // I2C TX, start condition
}

void MyWire::write(const unsigned char Data)
{
    UCB0TXBUF = Data;
    __bis_SR_register(LPM0_bits + GIE);  // go to sleep
}

void MyWire::endTransmission(void)
{
   // UCB0CTL1 |= UCTR + UCTXSTP;
   UCB0CTLW0 |= UCTXSTP;           // I2C stop condition
   UCB0IFG &= ~UCTXIFG;            // Clear USCI_B0 TX int flag  
   UCB0IE &= ~UCTXIE0;  //| UCNACKIE | UCBCNTIE; 
}

#pragma vector = USCI_B0_VECTOR
__interrupt void MyWire::USCIB0_ISR(void)
{
  switch(UCB0IV)
  {
        case USCI_NONE: break;                        // Vector 0: No interrupts break;
        case USCI_I2C_UCALIFG: break;
        case USCI_I2C_UCNACKIFG: break;               // Vector 4: NACKIFG break;
            //UCB0CTL1 |= UCTXSTT;                      //resend start if NACK
         // break;                                      
        case USCI_I2C_UCSTTIFG: break;                // Vector 6: STTIFG break;
        case USCI_I2C_UCSTPIFG: break;                // Vector 8: STPIFG break;
        case USCI_I2C_UCRXIFG3: break;                // Vector 10: RXIFG3 break;
        case USCI_I2C_UCTXIFG3: break;                // Vector 14: TXIFG3 break;
        case USCI_I2C_UCRXIFG2: break;                // Vector 16: RXIFG2 break;
        case USCI_I2C_UCTXIFG2: break;                // Vector 18: TXIFG2 break;
        case USCI_I2C_UCRXIFG1: break;                // Vector 20: RXIFG1 break;
        case USCI_I2C_UCTXIFG1: break;                // Vector 22: TXIFG1 break;
        case USCI_I2C_UCRXIFG0: break;                // Vector 24: RXIFG0 break;
        case USCI_I2C_UCTXIFG0:
            UCB0IFG &= ~UCTXIFG;                      // Clear USCI_B0 TX int flag
            __bic_SR_register_on_exit(LPM0_bits);     // Exit LPM0  Wake up
          break;                                      // Vector 26: TXIFG0 break;
        case USCI_I2C_UCBCNTIFG: break;               // Vector 28: BCNTIFG
        case USCI_I2C_UCCLTOIFG: break;               // Vector 30: clock low timeout
        case USCI_I2C_UCBIT9IFG: break;               // Vector 32: 9th bit
        default: break;
  }
    
}

void MyWire::MyDelay(void)
{
	TA0CCTL0 |= CCIE;                     // TACCR0 interrupt enabled
    TA0EX0 |= TAIDEX_3;                   // SMCLK/8/4 = 31250 Hz
     TA0CCR0 = 31250;                      // 1 per second 
    TA0CTL = TASSEL_2 | MC_1 | ID_3;             // SMCLK/8 = 125K , UP mode 
    
    // go to Standby 
    __bis_SR_register(LPM0_bits | GIE); // go to sleep
}    

// Timer A0 interrupt service routine
#pragma vector = TIMER0_A0_VECTOR
__interrupt void MyWire::Timer_A_ISR (void)
{
   // P1OUT ^= BIT0;
   __bic_SR_register_on_exit(LPM0_bits);   // Exit LPM0  Wake up
    
}
