//#include <Wire.h>
/*  Replace wire libaray
 * 
 */
#include <msp430.h>
#include "MyWire.h"

void OLED_init();

//~ DEFINES ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Some defines for the SSD1306 controller driving a 128x64 resolution OLED display
// PART     - http://www.simplelabs.co.in/content/96-blue-i2c-oled-module
// DATASHEET  - https://www.adafruit.com/datasheets/SSD1306.pdf

// The Slave Address (SLA) of the OLED controller - SSD1306 - is 0x3C 
// The LSB is supposed to be the mode. Since we are only going to WRITE to the OLED, the LSB is going to be 0
// SLA (0x3C) + WRITE_MODE (0x00) =  0x78 (0b01111000)
#define OLED_I2C_ADDRESS   0x3C

// The SSD1306 datasheet (pg.20) says that a control byte has to be sent before sending a command
// Control byte consists of 
// bit 7    : Co   : Continuation bit - If 0, then it assumes all the next bytes are data (no more control bytes).
//        :    You can send a stream of data, ie: gRAM dump - if Co=0
//        :        For Command, you'd prolly wanna set this - one at a time. Hence, Co=1 for commands
//        :    For Data stream, Co=0 :)
// bit 6      : D/C# : Data/Command Selection bit, Data=1/Command=0
// bit [5-0]  : lower 6 bits have to be 0
#define OLED_CONTROL_BYTE_CMD_SINGLE  0x80
#define OLED_CONTROL_BYTE_CMD_STREAM  0x00
#define OLED_CONTROL_BYTE_DATA_STREAM 0x40

// Fundamental commands (pg.28)
#define OLED_CMD_SET_CONTRAST     0x81  // follow with 0x7F
#define OLED_CMD_DISPLAY_RAM      0xA4
#define OLED_CMD_DISPLAY_ALLON      0xA5
#define OLED_CMD_DISPLAY_NORMAL     0xA6
#define OLED_CMD_DISPLAY_INVERTED     0xA7
#define OLED_CMD_DISPLAY_OFF      0xAE
#define OLED_CMD_DISPLAY_ON       0xAF

// Addressing Command Table (pg.30)
#define OLED_CMD_SET_MEMORY_ADDR_MODE 0x20  // follow with 0x00 = HORZ mode = Behave like a KS108 graphic LCD
#define OLED_CMD_SET_COLUMN_RANGE   0x21  // can be used only in HORZ/VERT mode - follow with 0x00 + 0x7F = COL127
#define OLED_CMD_SET_PAGE_RANGE     0x22  // can be used only in HORZ/VERT mode - follow with 0x00 + 0x07 = PAGE7

// Hardware Config (pg.31)
#define OLED_CMD_SET_DISPLAY_START_LINE 0x40
#define OLED_CMD_SET_SEGMENT_REMAP    0xA1  
#define OLED_CMD_SET_MUX_RATIO      0xA8  // follow with 0x3F = 64 MUX
#define OLED_CMD_SET_COM_SCAN_MODE    0xC8  
#define OLED_CMD_SET_DISPLAY_OFFSET   0xD3  // follow with 0x00
#define OLED_CMD_SET_COM_PIN_MAP    0xDA  // follow with 0x12

// Timing and Driving Scheme (pg.32)
#define OLED_CMD_SET_DISPLAY_CLK_DIV  0xD5  // follow with 0x80
#define OLED_CMD_SET_PRECHARGE      0xD9  // follow with 0x22
#define OLED_CMD_SET_VCOMH_DESELCT    0xDB  // follow with 0x30

// Charge Pump (pg.62)
#define OLED_CMD_SET_CHARGE_PUMP    0x8D  // follow with 0x14

// NOP
#define OLED_CMD_NOP          0xE3

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// Box pattern
uint8_t pattern1[] = {
  0x00,0x7E,0x42,0x42,0x42,0x42,0x7E,0x00,
  0x00,0x7E,0x42,0x42,0x42,0x42,0x7E,0x00
  };

// Cross weave pattern
uint8_t pattern2[] = {
  0x81,0x42,0x24,0x18,0x18,0x24,0x42,0x81,
  0x81,0x42,0x24,0x18,0x18,0x24,0x42,0x81
  };

MyWire WTx;

int main()
{                
  // Init the OLED controller
  
    OLED_init();



  // I2C
  WTx.beginTransmission(OLED_I2C_ADDRESS);
  WTx.write(OLED_CONTROL_BYTE_CMD_STREAM);
  WTx.write(OLED_CMD_SET_COLUMN_RANGE);
  WTx.write(0x00);
  WTx.write(0x7F);
  WTx.write(OLED_CMD_SET_PAGE_RANGE);
  WTx.write(0);
  WTx.write(0x07);
  WTx.endTransmission();

  for(uint16_t i=0;i<1024;i++){
    WTx.beginTransmission(OLED_I2C_ADDRESS);
    WTx.write(OLED_CONTROL_BYTE_DATA_STREAM);
      for (uint8_t x=0; x<16; x++) {  
      // WTx.write(0b11000001);  
      // WTx.write(0x81);  
      // WTx.write(0x02);
     // WTx.write(pattern1[x]);
       WTx.write(pattern2[x]); 
      i++;
      }
      i--;
      WTx.endTransmission();   
  }

    delay(5000);
  
}


void OLED_init() {
  // Init the I2C interface (pins A4 and A5 on the Arduino Uno board) in Master Mode.
  WTx.begin();
  // keywords:
  // SEG = COL = segment = column byte data on a page
  // Page = 8 pixel tall row. Has 128 SEGs and 8 COMs
  // COM = row

  // Begin the I2C comm with SSD1306's address (SLA+Write)
  WTx.beginTransmission(OLED_I2C_ADDRESS);

  // Tell the SSD1306 that a command stream is incoming
  WTx.write(OLED_CONTROL_BYTE_CMD_STREAM);

  // Follow instructions on pg.64 of the dataSheet for software configuration of the SSD1306
  // Turn the Display OFF
  WTx.write(OLED_CMD_DISPLAY_OFF);
  // Set mux ration tp select max number of rows - 64
  WTx.write(OLED_CMD_SET_MUX_RATIO);
  WTx.write(0x3F);
  // Set the display offset to 0
  WTx.write(OLED_CMD_SET_DISPLAY_OFFSET);
  WTx.write(0x00);
  // Display start line to 0
  WTx.write(OLED_CMD_SET_DISPLAY_START_LINE);
  
  // Mirror the x-axis. In case you set it up such that the pins are north.
  // WTx.write(0xA0); - in case pins are south - default
  WTx.write(OLED_CMD_SET_SEGMENT_REMAP);
    
  // Mirror the y-axis. In case you set it up such that the pins are north.
  // WTx.write(0xC0); - in case pins are south - default
  WTx.write(OLED_CMD_SET_COM_SCAN_MODE);
    
  // Default - alternate COM pin map
  WTx.write(OLED_CMD_SET_COM_PIN_MAP);
  WTx.write(0x12);
  // set contrast
  WTx.write(OLED_CMD_SET_CONTRAST);
  WTx.write(0x7F);
  // Set display to enable rendering from GDDRAM (Graphic Display Data RAM)
  WTx.write(OLED_CMD_DISPLAY_RAM);
  // Normal mode!
  WTx.write(OLED_CMD_DISPLAY_NORMAL);
  // Default oscillator clock
  WTx.write(OLED_CMD_SET_DISPLAY_CLK_DIV);
  WTx.write(0x80);
  // Enable the charge pump
  WTx.write(OLED_CMD_SET_CHARGE_PUMP);
  WTx.write(0x14);
  // Set precharge cycles to high cap type
  WTx.write(OLED_CMD_SET_PRECHARGE);
  WTx.write(0x22);
  // Set the V_COMH deselect volatage to max
  WTx.write(OLED_CMD_SET_VCOMH_DESELCT);
  WTx.write(0x30);
  // Horizonatal addressing mode - same as the KS108 GLCD
  WTx.write(OLED_CMD_SET_MEMORY_ADDR_MODE);
  WTx.write(0x00);
  // Turn the Display ON
  WTx.write(OLED_CMD_DISPLAY_ON);

  // End the I2C comm with the SSD1306
  WTx.endTransmission();
}
